/** \file main.c
 *
 * \brief Timer interrupt.
 *
 * This example program switches the LED on and off using a timer interrupt.  
 *
 * The relevant registers for the timer are:
 * - CTRLA: Timer Control Register A
 *   - ENABLE: Clock Enable
 *   - CLKSEL: Clock Select (Prescaler)
 * - INTCTRL: Timer Interrupt Control Register 
 *   -  OVF: Overflow Interrupt Enable
 * - INTFLAGS: Timer Interrupt Flag Register
 *   -  OVF: Overflow Interrupt Flag
 * - PER: Timer Period Register
 *
 * The following register is used to enable the timer interrupt:
 * - CPU_SREG: CPU Status Register
 *   - I: Global Interrupt Enable
 * The value for PER is calculated by PER = F_CPU/(n*TPS) - 1 where
 * - n: Prescale value
 * - TPS: Ticks per seconds
 * - F_CPU: System frequency
 *
 * Please see modules 4.3.2 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-20
 *
 * \par History
 *      Created: 2018-07-20
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include "board.h"

// We want the LED to blink with a frequency of 1 Hz. Blinking means that half
// of the time the LED is on and half of the time the LED is off. Therefore we
// need 2 interrupts per second to toggle the LED. We use a prescaler of 64. So
// we need a PER of 26041 (n=64, TPS=2, F_CPU=2000000). We use timer TCA0. As the
// job is done by the interrupt service routine the super loop has nothing to do.
int main(void)
{
    // Initialize the LED.
    LED_PORT.DIR = LED_PIN_bm;
    LED_PORT.OUT = LED_PIN_bm;

	// Initialize the timer.
	TCA0.SINGLE.PER = 26041;
    TCA0.SINGLE.INTCTRL = TCA_SINGLE_OVF_bm;
	TCA0.SINGLE.CTRLA = TCA_SINGLE_CLKSEL_DIV64_gc | TCA_SINGLE_ENABLE_bm;
	
	// Enable interrupts.
    CPU_SREG |= CPU_I_bm;

    while(1)
    {
        // Nothing to do.
    }
}

/***** Timer/counter overflow interrupt *****/
// This interrupt service routine gets called two times per second.
ISR(TCA0_OVF_vect)
{
	// Toggle the LED.
	LED_PORT.OUT ^= LED_PIN_bm;
	// We have to clear the interrupt flag. Surprisingly enough
	// it is cleared by writing a '1' to its bit location. 
	TCA0.SINGLE.INTFLAGS |= TCA_SINGLE_OVF_bm;
}
