/** \file main.c
 *
 * \brief USART interrupt.
 *
 * This example program receives characters from the console and sends them back.
 *
 * The relevant registers are:
 * - CTRLA: USART Control Register A
 *   - RXCINTLVL: Receive Complete Interrupt Level
 *
 * The following register is used to enable the timer interrupt:
 * - CPU_SREG: CPU Status Register
 *   - I: Global Interrupt Enable
 *
 * \note
 * This program implements example '04. USART' of the peripherals course with the
 * aid of a receive complete interrupt. Only the interrupt part is explained here.
 * Please see the peripherals course for more information on the USART part.
 *
 * \warning
 * This program demonstrates the use of the receive complete interrupt.
 * It is not correct as there is a race condition. Race conditions are
 * discussed in the next module and not taken into account here. Please
 * see the advanced example for a correct version of the program.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-23
 *
 * \par History
 *      Created: 2018-07-23
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

#define BUFFER_FULL 1
#define BUFFER_EMPTY 0

// Variables which are changed by interrupts must be declared as volatile.
volatile uint8_t buffer;
volatile uint8_t status = BUFFER_EMPTY;

// We want to send back the characters which we receive via the USART interface.
// We use the receive complete interrupt to handle the reception and store the
// character received in a buffer. If the buffer is full we send it back in
// the super loop.
int main(void)
{
    // Initialize the USART pins.
	VCOM_PORT.OUT |= VCOM_TX_PIN_bm;
	VCOM_PORT.DIR |= VCOM_TX_PIN_bm;
	VCOM_PORT.DIR &= ~VCOM_RX_PIN_bm;

    // Initialize USART and enable RXC interrupt.
    VCOM_MODULE.BAUD = VCOM_BAUDRATE;
	VCOM_MODULE.CTRLC = VCOM_CONFIG;
	VCOM_MODULE.CTRLA = USART_RXCIE_bm;
	VCOM_MODULE.CTRLB = (USART_RXEN_bm | USART_TXEN_bm);

	// Enable interrupts.
	CPU_SREG |= CPU_I_bm;

	while(1)
	{
		// Echo data if buffer is full.
		if (status == BUFFER_FULL) {
			// Wait until data register is empty.
			while(!(VCOM_MODULE.STATUS & USART_DREIF_bm));
			// Send character stored in buffer.
			VCOM_MODULE.TXDATAL = buffer;
			// The buffer is now empty again.
            status = BUFFER_EMPTY;
		}
	}
}

/*****  USART receive complete interrupt *****/
// This interrupt service routine gets called when new data has been received.
ISR(VCOM_RXC_vect)
{
	// Store character in buffer.
	buffer = VCOM_MODULE.RXDATAL;
	// Say that buffer is full.
	status = BUFFER_FULL;
	// We have to clear the interrupt flag. Surprisingly enough
	// it is cleared by writing a '1' to its bit location.
	VCOM_MODULE.STATUS |= USART_RXCIF_bm;
}
