/** \file board.h
 *
 * \brief Header file for the ATmega4809 Xplained Pro evaluation board.
 *
 * This header provides constants for the following components on the board:
 *
 * * \ref headers
 * * \ref mikrobus
 * * \ref led
 * * \ref button
 * * \ref atecc
 * * \ref vcom
 * * \ref dgi
 * * \ref oscillators
 *
 * \author    Wolfgang Neff
 * \version   1.2
 * \date      2019-02-09
 *
 * \par History
 *      Created: 2018-07-14 \n
 *      Modified: 2018-07-23 \n
 *      Modified: 2019-02-09
 *
 * \note
 *      **VCOM:** Line parameters for the virtual COM port: 115200 8N1. \n
 *      **F_CPU:** Default 3333333. Use default or set it via Project/Properties/Toolchain/Compiler/Symbols. \n
 *      **FreeROTS:** TCB3 used as tick generator. Default: 1000 low level interrupts ticks per second.
 */

#ifndef ATMEGA4809_XPLAINED_PRO_H_
#define ATMEGA4809_XPLAINED_PRO_H_

/** @defgroup headers Header configuration
 <summary>Pin assignment of the extension headers.</summary>
@{
 | Pin | EXT1                     | EXT2              | EXT3                    |
 |-----|--------------------------|-------------------|-------------------------|
 |  1  |   -  (ID)                |  -  (ID)          |  -  (ID)                |
 |  2  |   -  (GND)               |  -  (GND)         |  -  (GND)               |
 |  3  |  PD2 (ADC0 AIN2)         | PD4 (ADC0 AIN4)   | PD6 (ADC0 AIN6)         |
 |  4  |  PD3 (ADC0 AIN3)         | PD5 (ADC0 AIN5)   | PD7 (ADC0 AIN7)         |
 |  5  |  PA2 (GPIO, USART0 XCK)  | PE0 (GPIO)        | PD0 (GPIO)              |
 |  6  |  PA3 (GPIO, USART0 XDIR) | PF6 (GPIO, RESET) | PD1 (GPIO)              |
 |  7  |  PC4 (TCA0 WO4)          | PB4 (TCA0 WO4)    | PF4 (TCB0 WO, TCA0 WO4) |
 |  8  |  PC5 (TCA0 WO5)          | PB5 (TCA0 WO5)    | PF5 (TCB1 WO, TCA0 WO5) |
 |  9  |  PC6 (GPIO)              | PB2 (GPIO)        | PE2 (GPIO)              |
 | 10  |  PC7 (GPIO)              | PB3 (GPIO)        | PE3 (GPIO)              |
 | 11  |  PC2 (TWI0 SDA)          | PC2 (TWI0 SDA)    | PC2 (TWI0 SDA)          |
 | 12  |  PC3 (TWI0 SCL)          | PC3 (TWI0 SCL)    | PC3 (TWI0 SCL)          |
 | 13  |  PA1 (USART0 RxD)        | PC1 (USART1 RxD)  | PB1 (USART3 RxD)        |
 | 14  |  PA0 (USART0 TxD)        | PC0 (USART1 TxD)  | PB0 (USART3 TxD)        |
 | 15  |  PA7 (SPI0 SS)           | PE1 (GPIO)        | PF2 (GPIO)              |
 | 16  |  PA4 (SPI0 MOSI)         | PA4 (SPI0 MOSI)   | PA4 (SPI0 MOSI)         |
 | 17  |  PA5 (SPI0 MISO)         | PA5 (SPI0 MISO)   | PA5 (SPI0 MISO)         |
 | 18  |  PA6 (SPI0 SCK)          | PA6 (SPI0 SCK)    | PA6 (SPI0 SCK)          |
 | 19  |   -  (GND)               |  -  (GND)         |  -  (GND)               |
 | 20  |   -  (VCC)               |  -  (VCC)         |  -  (VCC)               |
@} */

/** @defgroup mikrobus mikroBUS 
 <summary>Definitions for the mikroBUS socket.</summary>
@{
 | Pin | Port | Function  | mikroBUS | mikroBUS | Function   | Port | Pin |
 |-----|------|-----------|----------|----------|------------|------|-----|
 |  1  | PD6  | ADC0 AIN7 |  AN      |  PWM     | TCB0 WO    | PF4  |  1  |
 |  2  | PD0  | GPIO      |  RST     |  INT     | GPIO       | PE2  |  2  |
 |  3  | PF2  | GPIO      |  CS      |  RX      | USART3 RxD | PB1  |  3  |
 |  4  | PA6  | SPI0 SCK  |  SCK     |  TX      | USART3 TxD | PB0  |  4  |
 |  5  | PA5  | SPI0 MISO |  MISO    |  SCL     | TWI0 SCL   | PC3  |  5  |
 |  6  | PA4  | SPI0 MOSI |  MOSI    |  SDA     | TWI0 SDA   | PC2  |  6  |
 |  7  |  -   |  -        |  3V3     |  5V0     |  -         |  -   |  7  |
 |  8  |  -   |  -        |  GND     |  GND     |  -         |  -   |  8  |
*/

/****** mikroBUS Socket ******/
#define MIKROBUS_AN_PORT PORTD
#define MIKROBUS_AN_PIN_bp PIN6_bp
#define MIKROBUS_AN_PIN_bm PIN6_bm

#define MIKROBUS_RST_PORT PORTD
#define MIKROBUS_RST_PIN_bp PIN0_bp
#define MIKROBUS_RST_PIN_bm PIN0_bm

#define MIKROBUS_CS_PORT PORTF
#define MIKROBUS_CS_PIN_bp PIN2_bp
#define MIKROBUS_CS_PIN_bm PIN2_bm

#define MIKROBUS_SCK_PORT PORTA
#define MIKROBUS_SCK_PIN_bp PIN6_bp
#define MIKROBUS_SCK_PIN_bm PIN6_bm

#define MIKROBUS_MISO_PORT PORTA
#define MIKROBUS_MISO_PIN_bp PIN6_bp
#define MIKROBUS_MISO_PIN_bm PIN6_bm

#define MIKROBUS_MOSI_PORT PORTA
#define MIKROBUS_MOSI_PIN_bp PIN6_bp
#define MIKROBUS_MOSI_PIN_bm PIN6_bm

#define MIKROBUS_PWM_PORT PORTF
#define MIKROBUS_PWM_PIN_bp PIN4_bp
#define MIKROBUS_PWM_PIN_bm PIN4_bm

#define MIKROBUS_INT_PORT PORTE
#define MIKROBUS_INT_PIN_bp PIN2_bp
#define MIKROBUS_INT_PIN_bm PIN2_bm

#define MIKROBUS_RX_PORT PORTB
#define MIKROBUS_RX_PIN_bp PIN1_bp
#define MIKROBUS_RX_PIN_bm PIN1_bm

#define MIKROBUS_TX_PORT PORTB
#define MIKROBUS_TX_PIN_bp PIN0_bp
#define MIKROBUS_TX_PIN_bm PIN0_bm

#define MIKROBUS_SCL_PORT PORTC
#define MIKROBUS_SCL_PIN_bp PIN3_bp
#define MIKROBUS_SCL_PIN_bm PIN3_bm

#define MIKROBUS_SDA_PORT PORTC
#define MIKROBUS_SDA_PIN_bp PIN2_bp
#define MIKROBUS_SDA_PIN_bm PIN2_bm

#define MIKROBUS_SPI_PORT PORTA
#define MIKROBUS_SPI_MODULE SPI0
#define MIKROBUS_SPI_SCK_bm PIN6_bm
#define MIKROBUS_SPI_MISO_bm PIN5_bm
#define MIKROBUS_SPI_MOSI_bm PIN4_bm

#define MIKROBUS_USART_PORT PORTB
#define MIKROBUS_USART_MODULE USART3
#define MIKROBUS_USART_RX_bm PIN1_bm
#define MIKROBUS_USART_TX_bm PIN0_bm
#define MIKROBUS_USART_DRE_vect USART3_DRE_vect
#define MIKROBUS_USART_RXC_vect USART3_RXC_vect

#define MIKROBUS_I2C_PORT PORTC
#define MIKROBUS_I2C_MODULE TWI0
#define MIKROBUS_I2C_SCL_bm PIN3_bm
#define MIKROBUS_I2C_SDA_bm PIN2_bm

#define MIKROBUS_PWM_MODULE TCB0
/// @}

/** @defgroup led LED
 <summary>Definitions for the active low LED.</summary>
 <remarks>
 There is one yellow user programmable LED  on the board.
 Please consult the schematic for details. 
 \verbatim
  VCC_P3V3
     +
     |
    .-.
    | | 1k
    | |
    '-'
     |
     |
     V -> Yellow LED
     -
     |
     |
    PB5
 \endverbatim
 </remarks>
@{ */
/****** Active low LED ******/
#define LED_PORT PORTB
#define LED_PIN_bp PIN5_bp
#define LED_PIN_bm PIN5_bm
#define LED_PINCTRL LED_PORT.PIN5CTRL
/// @}

/** @defgroup button Mechanical Push-Button
 <summary>Definitions for the active low push-button.</summary>
 <remarks>
 There is one mechanical user programmable push-button on the board.
 The second push-button is either user programmable or a reset button.
 Please consult the schematic for details. 
 \verbatim
                                   PF6
                                    |
                                   .-.
                                   | | 100k
                                   | |
                                   '-'
                                    |
    PB2                             o--------------------.
     |                              |                    |
    .-.                            .-.                   |
    | | 39R                        | | 39R               |
    | |                            | |                   |
    '-'                            '-'                   |
     |                              |                    |
     |                              |                    |
   | o                            | o                    |
 |=|>   TS604VM1-035CR          |=|>   TS604VM1-035CR  ----- 4n7
   | o                            | o                  -----
     |                              |                    |
     |                              |                    |
    ===                            ===                  ===
    GND                            GND                  GND
 \endverbatim
 </remarks>
@{ */ 
/****** Active low push-button ******/
#define BUTTON_PORT BUTTON0_PORT
#define BUTTON_PIN_bp BUTTON0_PIN_bp
#define BUTTON_PIN_bm BUTTON0_PIN_bm
#define BUTTON_PINCTRL BUTTON0_PINCTRL

#define BUTTON0_PORT PORTB
#define BUTTON0_PIN_bp PIN2_bp
#define BUTTON0_PIN_bm PIN2_bm
#define BUTTON0_PINCTRL BUTTON_PORT.PIN2CTRL

#define BUTTON1_PORT PORTF
#define BUTTON1_PIN_bp PIN6_bp
#define BUTTON1_PIN_bm PIN6_bm
#define BUTTON1_PINCTRL BUTTON1_PORT.PIN6CTRL
/// @}

/** @defgroup atecc ATECC508A CryptoAuthentication Device 
 <summary>Definitions for the ATECC508A CryptoAuthentication device.</summary>
@{ */
/****** ATECC508A CryptoAuthentication Device ******/
#define ATECC_PORT PORTC
#define ATECC_MODULE TWI0
#define ATECC_SDA_PIN_bp PIN2_bp
#define ATECC_SDA_PIN_bm PIN2_bm
#define ATECC_SCL_PIN_bp PIN3_bp
#define ATECC_SCL_PIN_bm PIN3_bm
/// @}

/** @defgroup vcom Virtual COM Port 
 <summary>Definitions for the virtual COM port.</summary>
 <remarks>
 USART1 is connected with the virtual COM port.
 Its line configuration is 115200 8N1.
 </remarks>
@{ */
/****** Virtual COM Port ******/
#define VCOM_PORT PORTC
#define VCOM_MODULE USART1

#define VCOM_RX_PIN_bp PIN1_bp
#define VCOM_RX_PIN_bm PIN1_bm

#define VCOM_TX_PIN_bp PIN0_bp
#define VCOM_TX_PIN_bm PIN0_bm

#define VCOM_RXC_vect USART1_RXC_vect
#define VCOM_DRE_vect USART1_DRE_vect

#define VCOM_SPEED 115200
#define VCOM_BAUDRATE 116
#define VCOM_CONFIG (USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc)
/// @}

/** @defgroup dgi Data Gateway Interface  
 <summary>Definitions for the embedded debugger data gateway interface.</summary>
@{ */
/****** Data Gateway Interface ******/
#define DGI_SS_PORT PORTF
#define DGI_SS_PIN_bp PIN3_bp
#define DGI_SS_PIN_bm PIN3_bm

#define DGI_SPI_PORT PORTA
#define DGI_SPI_MODULE SPI0
#define DGI_MOSI_PIN_bp PIN4_bp
#define DGI_MOSI_PIN_bm PIN4_bm
#define DGI_MISO_PIN_bp PIN5_bp
#define DGI_MISO_PIN_bm PIN5_bm
#define DGI_SCK_PIN_bp PIN6_bp
#define DGI_SCK_PIN_bm PIN6_bm

#define DGI_I2C_PORT PORTC
#define DGI_I2C_MODULE TWI0
#define DGI_I2C_SDA_PIN_bp PIN2_bp
#define DGI_I2C_SDA_PIN_bm PIN2_bm
#define DGI_I2C_SCL_PIN_bp PIN3_bp
#define DGI_I2C_SCL_PIN_bm PIN3_bm
/// @}

/** @defgroup oscillators Oscillators
 <summary>Constants for the oscillators.</summary>
@{ */
/****** Internal oscillators ******/
#define OSC_INTERNAL_20MHZ 20000000L
#define OSC_INTERNAL_32kHZ 320000L

/****** System clocks ******/
#define CLK_MAIN_PRESCALER 6
#define CLK_CPU (OSC_INTERNAL_20MHZ/CLK_MAIN_PRESCALER)
#define CLK_PER CLK_CPU

/****** CPU clock frequency ******/
#ifndef F_CPU
#define F_CPU CLK_CPU
#warning "F_CPU initialized with default value"
#endif
/// @}

#endif /* ATMEGA4809_XPLAINED_PRO_H_ */
