/** \file main.c
 *
 * \brief Port interrupt.
 *
 * This example program checks if push-buttons are pressed.
 *
 * The relevant register is:
 * - PINnCTRL: Pin n Control Register
 *   - ISC: (Port Interrupt) Input/Sense Configuration
 *
 * We need two push-buttons for this example. Therefore we will use the reset
 * button as user programmable button. This is possible unless PF6 is configured
 * as a RESET pin. By default it is not configured that way.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-23
 *
 * \par History
 *      Created: 2018-07-23
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "board.h"

// We want to switch the LED on if button 0 is pressed and
// we want to switch it off if button 1 is pressed.
int main(void)
{
	// Initialize the LED.
    LED_PORT.DIR = LED_PIN_bm;
    LED_PORT.OUT = LED_PIN_bm;

    // Initialize push-buttons and enable interrupts.
	BUTTON0_PINCTRL = PORT_PULLUPEN_bm | PORT_ISC_FALLING_gc;
	BUTTON1_PINCTRL = PORT_PULLUPEN_bm | PORT_ISC_FALLING_gc;

	// Enable interrupts. 
	CPU_SREG |= CPU_I_bm;

	while(1)
	{
        // Nothing to do
	}
}

/***** Port B interrupt *****/
// This interrupt service routine gets called whenever PORTB interrupt is thrown.
ISR(PORTB_PORT_vect)
{
	// Switch the LED on.
	LED_PORT.OUT &= ~LED_PIN_bm;
	// We have to clear the interrupt flag. Surprisingly enough
	// it is cleared by writing a '1' to its bit location.
	BUTTON0_PORT.INTFLAGS |= BUTTON_PIN_bm;
}

/***** Port F interrupt *****/
// This interrupt service routine gets called whenever PORTF interrupt is thrown.
ISR(PORTF_PORT_vect)
{
	// Switch the LED off
	LED_PORT.OUT |= LED_PIN_bm;
	// We have to clear the interrupt flag. Surprisingly enough
	// it is cleared by writing a '1' to its bit location.
	BUTTON1_PORT.INTFLAGS |= BUTTON_PIN_bm;
}
